#include <sys/socket.h>
#include <linux/if_arp.h>

#include <features.h>    /* for the glibc version number */
#if __GLIBC__ >= 2 && __GLIBC_MINOR >= 1
    #include <netpacket/packet.h>
    #include <net/ethernet.h>     /* the L2 protocols */
#else
    #include <asm/types.h>
    #include <linux/if_packet.h>
    #include <linux/if_ether.h>   /* The L2 protocols */
#endif

#define BLEN 1500
#define TIPO_ARP htons(0x0806)
#define TIPO_IP  htons(0x0800)

int ricevi (int s,char * buff,unsigned short *prot);
void stampa_arp (unsigned char * buff,int len);

struct arp_packet {
    unsigned short hwtype;
    unsigned short prottype;
    unsigned char  hwsize;
    unsigned char  protsize;
    unsigned short op;
    unsigned char  sender_ha[6];
    unsigned char  sender_pa[4];
    unsigned char  receiver_ha[6];
    unsigned char  receiver_pa[4];
};

struct arp_table {
    unsigned char ip[4];
    unsigned char mac[6];
};

struct arp_table at[1000];
int at_n=0;
struct arp_packet *arp_pt;

int main () {
    int s,l;
    unsigned short prot;
    char buff[BLEN];

    s=socket(PF_PACKET,SOCK_DGRAM,htons(ETH_P_ALL));
    if (s==-1) {
        perror("Inizializzazione socket fallita");
        return -1;
    }//if

    while (1) {
	l=ricevi(s,buff,&prot);
	if (l<0)
	    exit(l);
	if (prot==TIPO_IP) {		/* routing */
	    ;
	}//if
	else if (prot==TIPO_ARP) {	/* memorizza informazioni */
	    stampa_arp(buff,l);
	    return 1;
	}//else if
    }//while
}//main

int ricevi (int s,char * buff,unsigned short *prot) {
    int len,
	from_len;
    struct sockaddr_ll from;
    s=socket(PF_PACKET,SOCK_DGRAM,htons(ETH_P_ALL));
    if (s==-1) {
        perror("Inizializzazione socket fallita");
	return -1;
    }//if

    from.sll_family=AF_PACKET;
    from.sll_protocol=htons(ETH_P_ALL);
    from.sll_ifindex=0;
    from.sll_hatype=ARPHRD_ETHER;
    from.sll_pkttype=PACKET_HOST;
    from.sll_halen=0;
    from_len=sizeof(struct sockaddr_ll);

    if ((len=recvfrom(s,buff,BLEN,0,(struct sockaddr *)&from,&from_len))==-1) {
        perror("recvfrom fallita");
        return -2;
    }//if

    *prot=from.sll_protocol;
    return len;
}//ricevi

void stampa_arp (unsigned char * buff,int len) {
    int i;
    for(i=0;i<len;i++) {
	if (i%4==0)
	    printf("\n\t");
	printf("%x(%d)\t",buff[i],buff[i]);
    }//for
    printf("\n\n");
    arp_pt=(struct arp_packet *)buff;
    printf("\tTipo operazione..: %d\n",htons(arp_pt->op));
    printf("\tIP sorgente......: %d.%d.%d.%d\n",arp_pt->sender_pa[0],arp_pt->sender_pa[1],
						arp_pt->sender_pa[2],arp_pt->sender_pa[3]);
    printf("\tIP destinazione..: %d.%d.%d.%d\n\n",arp_pt->receiver_pa[0],arp_pt->receiver_pa[1],
						  arp_pt->receiver_pa[2],arp_pt->receiver_pa[3]);
}//main

void insert_arp (unsigned char *ip,unsigned char *mac) {
    int i;
    for (i=0;i<4;i++)
	at[at_n].ip[i]=ip[i];
    for (i=0;i<6;i++)
	at[at_n].mac[i]=mac[i];
    at_n++;
}

int cerca_mac(unsigned char *ip,unsigned char *mac) {
    int i;
    for (i=0;i<at_n;i++)
	if (*((unsigned long *)ip) == *((unsigned long *)(at[i].ip))) {
	    for (j=0;j<at_n;j++)
		mac[j]=at[i].mac[j];
	    return 1;
	}//if
    return 0;
}
