#include <sys/socket.h>
#include <linux/if_arp.h>

#include <features.h>    /* for the glibc version number */
#if __GLIBC__ >= 2 && __GLIBC_MINOR >= 1
    #include <netpacket/packet.h>
    #include <net/ethernet.h>     /* the L2 protocols */
#else
    #include <asm/types.h>
    #include <linux/if_packet.h>
    #include <linux/if_ether.h>   /* The L2 protocols */
#endif

#define BLEN 1500
#define TIPO_ARP htons(0x0806)
#define TIPO_IP  htons(0x0800)

struct arp_packet {
    unsigned short hwtype;
    unsigned short prottype;
    unsigned char  hwsize;
    unsigned char  protsize;
    unsigned short op;
    unsigned char  sender_ha[6];
    unsigned char  sender_pa[4];
    unsigned char  receiver_ha[6];
    unsigned char  receiver_pa[4];
};

struct arp_table {
    unsigned char ip[4];
    unsigned char mac[6];
};

struct arp_table at[1000];
int    at_n=0;
struct arp_packet *arp_pt;
unsigned char *mac;
unsigned char *ip;

int  ricevi      (int s,char * buff,unsigned short *prot);
void insert_arp  (unsigned char *ip,unsigned char *mac);
int  elabora_arp (struct arp_packet *ap,unsigned char **ip,unsigned char **mac);
int  cerca_mac   (unsigned char *ip,unsigned char *mac);
void stampa_at   ();

int main () {
    int s,l;
    unsigned short prot;
    char buff[BLEN];

    s=socket(PF_PACKET,SOCK_DGRAM,htons(ETH_P_ALL));
    if (s==-1) {
        perror("Inizializzazione socket fallita");
        return -1;
    }//if

    while (1) {
        l=ricevi(s,buff,&prot);
        if (l<0)
            exit(l);
        if (prot==TIPO_IP) {            /* routing */
            decidi();
            inoltra();
        }//if
        else if (prot==TIPO_ARP) {      /* memorizza informazioni */
            if (elabora_arp((struct arp_packet *)buff,&ip,&mac)==1) { /* estrae la coppia hw/prot address */
                if (cerca_mac(ip,mac)==0)   /* non trovato */
                    insert_arp(ip,mac);
                system("clear");
                stampa_at();            /* stampa arp table */
                //return 1;
            }//if
        }//else if
    }//while
}//main

int ricevi (int s,char * buff,unsigned short *prot) {
    int len,
    from_len;
    struct sockaddr_ll from;

    from.sll_family=AF_PACKET;
    from.sll_protocol=htons(ETH_P_ALL);
    from.sll_ifindex=0;
    from.sll_hatype=ARPHRD_ETHER;
    from.sll_pkttype=PACKET_HOST;
    from.sll_halen=0;
    from_len=sizeof(struct sockaddr_ll);

    if ((len=recvfrom(s,buff,BLEN,0,(struct sockaddr *)&from,&from_len))==-1) {
        perror("recvfrom fallita");
        return -2;
    }//if

    *prot=from.sll_protocol;
    return len;
}//ricevi

void insert_arp (unsigned char *ip,unsigned char *mac) {
    int i;
    for (i=0;i<4;i++)
    at[at_n].ip[i]=ip[i];
    for (i=0;i<6;i++)
    at[at_n].mac[i]=mac[i];
    at_n++;
}//insert_arp

int cerca_mac (unsigned char *ip,unsigned char *mac) {
    int i,j;
    for (i=0;i<at_n;i++)
    if (*((unsigned long *)ip) == *((unsigned long *)(at[i].ip))) {
        for (j=0;j<at_n;j++)
        mac[j]=at[i].mac[j];
        return 1;
    }//if
    return 0;
}//cerca_mac

int elabora_arp (struct arp_packet * ap,unsigned char **ip,unsigned char **mac) {
    if (ap->sender_pa[0]||ap->sender_pa[1]||ap->sender_pa[2]||ap->sender_pa[3]) {
        /* invio della coppia mac+ip */
        *ip=ap->sender_pa;
        *mac=ap->sender_ha;
        return 1;
    }//if

    if (ap->receiver_pa[0]||ap->receiver_pa[1]||ap->receiver_pa[2]||ap->receiver_pa[3]) {
        /* invio della coppia mac+ip */
        *ip=ap->receiver_pa;
        *mac=ap->receiver_ha;
        return 1;
    }//if
}//elabora_arp

void stampa_at() {
    int i;
    printf("\n-------------------------------------------------------------\n");
    for (i=0;i<at_n;i++) {
        printf("%3d) ",i);
        printf("%3d.%3d.%3d.%3d",at[i].ip[0],at[i].ip[1],at[i].ip[2],at[i].ip[3]);
        printf("    <-->    ");
        printf("%2x : %2x : %2x : %2x : %2x : %2x\n",at[i].mac[0],at[i].mac[1],at[i].mac[2],at[i].mac[3],at[i].mac[4],at[i].mac[5]);
    }//for
    printf("-------------------------------------------------------------\n");
}//stampa_at