#include <sys/socket.h>
#include <linux/if_arp.h>

#include <features.h>    /* for the glibc version number */
#if __GLIBC__ >= 2 && __GLIBC_MINOR >= 1
    #include <netpacket/packet.h>
    #include <net/ethernet.h>     /* the L2 protocols */
#else
    #include <asm/types.h>
    #include <linux/if_packet.h>
    #include <linux/if_ether.h>   /* The L2 protocols */
#endif

#define BLEN 1500
#define TIPO_ARP htons(0x0806)
#define TIPO_IP  htons(0x0800)

struct route_entry {
	unsigned int dest;
	unsigned int mask;
	unsigned int nextip;
	unsigned int ifn;		//interface number
};

struct arp_packet {
    unsigned short hwtype;
    unsigned short prottype;
    unsigned char  hwsize;
    unsigned char  protsize;
    unsigned short op;
    unsigned char  sender_ha[6];
    unsigned char  sender_pa[4];
    unsigned char  receiver_ha[6];
    unsigned char  receiver_pa[4];
};

struct arp_table {
    unsigned char ip[4];
    unsigned char mac[6];
};

struct route_entry rt[10];
	/* 147.162.35.0 : sono gia' arrivati, sull'interfaccia 2
	   10.1.0.0		: sono gia' arrivati, sull'interfaccia 3 */

int rt_n;
void init_rt() {	//iniziallizzazione della tabella di route
	//riga #0	-	il pkt appartine alla rete attuale
	((unsigned char *)(&rt[0].dest))[0]=147;
	((unsigned char *)(&rt[0].dest))[1]=162;
	((unsigned char *)(&rt[0].dest))[2]=35;
	((unsigned char *)(&rt[0].dest))[3]=0;
	rt[0].mask=htonl(0xFFFFFF00);	/* maschera 255.255.255.0 */
	rt[0].nextip=0;					/* quello che sta scritto nel pacchetto */
	rt[0].ifn=2;

	//riga #1	-	spediscilo al outer dell'altra rete
	((unsigned char *)(&rt[1].dest))[0]=10;
	((unsigned char *)(&rt[1].dest))[1]=1;
	((unsigned char *)(&rt[1].dest))[2]=1;
	((unsigned char *)(&rt[1].dest))[3]=0;
	rt[1].mask=htonl(0xFFFFFF00);	/* maschera 255.255.255.0 */
	rt[1].nextip=0;					/* quello che sta scritto nel pacchetto */
	rt[1].ifn=3;

	//riga #2	- default
	rt[2].dest=0;					/* destinazione */
	rt[2].mask=0;					/* maschera 0.0.0.0 */
	((unsigned char *)(&rt[2].nextip))[0]=147;
	((unsigned char *)(&rt[2].nextip))[1]=162;
	((unsigned char *)(&rt[2].nextip))[2]=35;
	((unsigned char *)(&rt[2].nextip))[3]=254;
	rt[2].ifn=2;

	rt_n=3;
}//init_route_table

struct arp_table at[1000];
int    at_n=0;
struct arp_packet *arp_pt;
unsigned char *mac;
unsigned char *ip;

int  ricevi      (int s,char * buff,unsigned short *prot);
void insert_arp  (unsigned char *ip,unsigned char *mac);
int  elabora_arp (struct arp_packet *ap,unsigned char **ip,unsigned char **mac);
int  cerca_mac   (unsigned char *ip,unsigned char *mac);
void stampa_at   ();
int decidi (unsigned char *b,int *ifn,unsigned int *nextip);

int main () {
    unsigned short prot;
	unsigned int nextip;
    int  s,l;
    char buff[BLEN];
	int  interface;					/* interfaccia sulla quale spedire il pacchetto	*/
	unsigned char nextmac[6];		/* mac addr del prossimo hop */

	init_rt();	/* inizializzazione della tabella di routing */

    s=socket(PF_PACKET,SOCK_DGRAM,htons(ETH_P_ALL));
    if (s==-1) {
        perror("Inizializzazione socket fallita");
        return -1;
    }//if

    while (1) {
        l=ricevi(s,buff,&prot);
        if (l<0)
            exit(l);
        if (prot==TIPO_IP) {            /* routing */
            if (decidi(buff,&interface,&nextip)==0)
				if (cerca_mac((unsigned char *)&nextip,nextmac)==0)
					invia_arp(s,nextip,interface);
				else
					inoltra(s,interface,nextmac);
        }//if
        else if (prot==TIPO_ARP) {      /* memorizza informazioni */
            if (elabora_arp((struct arp_packet *)buff,&ip,&mac)==1) { /* estrae la coppia hw/prot address */
                if (cerca_mac(ip,mac)==0)   /* non trovato */
                    insert_arp(ip,mac);
                system("clear");
                stampa_at();            /* stampa arp table */
                //return 1;
            }//if
        }//else if
    }//while
}//main

int ricevi (int s,char * buff,unsigned short *prot) {
    int len,
    from_len;
    struct sockaddr_ll from;

    from.sll_family=AF_PACKET;
    from.sll_protocol=htons(ETH_P_ALL);
    from.sll_ifindex=0;
    from.sll_hatype=ARPHRD_ETHER;
    from.sll_pkttype=PACKET_HOST;
    from.sll_halen=0;
    from_len=sizeof(struct sockaddr_ll);

    if ((len=recvfrom(s,buff,BLEN,0,(struct sockaddr *)&from,&from_len))==-1) {
        perror("recvfrom fallita");
        return -2;
    }//if

    *prot=from.sll_protocol;
    return len;
}//ricevi

void insert_arp (unsigned char *ip,unsigned char *mac) {
    int i;
    for (i=0;i<4;i++)
    at[at_n].ip[i]=ip[i];
    for (i=0;i<6;i++)
    at[at_n].mac[i]=mac[i];
    at_n++;
}//insert_arp

int cerca_mac (unsigned char *ip,unsigned char *mac) {
    int i,j;
    for (i=0;i<at_n;i++)
    if (*((unsigned long *)ip) == *((unsigned long *)(at[i].ip))) {
        for (j=0;j<at_n;j++)
        mac[j]=at[i].mac[j];
        return 1;
    }//if
    return 0;
}//cerca_mac

int elabora_arp (struct arp_packet * ap,unsigned char **ip,unsigned char **mac) {
    if (ap->sender_pa[0]||ap->sender_pa[1]||ap->sender_pa[2]||ap->sender_pa[3]) {
        /* invio della coppia mac+ip */
        *ip=ap->sender_pa;
        *mac=ap->sender_ha;
        return 1;
    }//if

    if (ap->receiver_pa[0]||ap->receiver_pa[1]||ap->receiver_pa[2]||ap->receiver_pa[3]) {
        /* invio della coppia mac+ip */
        *ip=ap->receiver_pa;
        *mac=ap->receiver_ha;
        return 1;
    }//if
}//elabora_arp

void stampa_at() {
    int i;
    printf("\n-------------------------------------------------------------\n");
    for (i=0;i<at_n;i++) {
        printf("%3d) ",i);
        printf("%3d.%3d.%3d.%3d",at[i].ip[0],at[i].ip[1],at[i].ip[2],at[i].ip[3]);
        printf("    <-->    ");
        printf("%2x : %2x : %2x : %2x : %2x : %2x\n",at[i].mac[0],at[i].mac[1],at[i].mac[2],at[i].mac[3],at[i].mac[4],at[i].mac[5]);
    }//for
    printf("-------------------------------------------------------------\n");
}//stampa_at

int decidi (unsigned char *b,int *ifn,unsigned int *nextip) {
	unsigned int *dest;
	int trovato=0,i;

	dest=(unsigned int *)b+4;	/* oppure:   dest=&(((unsigned int *)b)[4]); */
	for(i=0;i<rt_n&&trovato==0;i++) {
		if(*dest & rt[i].mask == rt[i].dest & rt[i].mask) {
			trovato=1;
			*ifn=rt[i].ifn;
			*nextip=rt[i].nextip;
			if (*nextip==0)
				*nextip=*dest;
		}//if
	}//for
	if (trovato==0)
		return 1;		//errore
	return 0;
}//decidi

void invia_arp(int s,unsigned char *ip,int ifn) {
	struct sockaddr_ll to;
	struct arp_packet ap;

    to.sll_family=AF_PACKET;
    to.sll_protocol=TIPO_ARP;
    to.sll_ifindex=ifn;
    to.sll_hatype=ARPHRD_ETHER;	/* tipo di indirizzo */
    to.sll_addr[0]=to.sll_addr[1]=to.sll_addr[2]=to.sll_addr[3]=to.sll_addr[4]=to.sll_addr[5]=255;
	to.halen=6;

	sendto(s,   ,&to,sizeof(struct sockaddr_ll));

	ap.hwtype=1;
	ap.prottype=TIPO_IP;
	ap.hwsize=6;
	ap.protsize=4;
	ap.op=htons(1);
	ap.sender_ha[0]=ap.sender_ha[1]=ap.sender_ha[2]=ap.sender_ha[3]=ap.sender_ha[4]=ap.sender_ha[5]=0;
	ap.sender_pa[0];
	ap.receiver_ha[0];
	ap.receiver_pa[0];
}//invia_arp

void inoltra(int s,interface,nextmac) {
}//inoltra
