/* Esame.c   Meccanismo standard per PING  */


/* passi:
   1 - costruzione del pacchetto IP + ICMP (tipo 8)
   2 - invio del pacchetto costruito
   
   3 - cattura dei pacchetti IP ricevuti
   4 - trovo pacchetti ICMP di risposta (tipo 0)
   5 - scrivere nodo x.x.x.x raggiunto
*/


/* include o meno dei pacchetti a seconda del compilatore  *
#include <sys/socket.h>
#include <features.h>    /* for the glibc version number */
#if __GLIBC__ >= 2 && __GLIBC_MINOR >= 1
 #include <netpacket/packet.h>
 #include <net/ethernet.h>     /* the L2 protocols */
#else
 #include <asm/types.h>
 #include <linux/if_packet.h>
 #include <linux/if_ether.h>   /* The L2 protocols */
#endif

#include <sys/types.h>
#include <sys/socket.h>
#include <string.h>


#define  ETH_SIZE  1500

// struttura per definire il bit-stream da ricevere/inviare a livello 2
struct sockaddr_ll from;  // da dove ricevo il pacchetto


// definisco la struct per leggere un bit-stream con il formato del pacchetto ICMP
struct pacc_icmp {
  unsigned char  type  ;    
  unsigned char  code  ;
  short    int   chk   ;
  short    int   ident ;
  short    int   seqnum;      // 16 bit
  unsigned char  dati[4];
  } ;


// definisco la struct per leggere un bit-stream con il formato del pacchetto IP
 struct pacc_ip {
   unsigned  char ver_len       ;  // versione + lung. preambolo  8 bit  
   unsigned  char tos           ;  // tipo di servizio            8 bit 
   short     int  tot_len       ;  // lung. totale pacchetto     16 bit
   short     int  identif       ;  // identif. per frammentaz    16 bit
   short     int  ffrag         ;
   unsigned  char ttl           ;  // time to live                8 bit 
   unsigned  char prot          ;  // protocollo (TCP/UDP)        8 bit
   short     int  hfcs          ;  // header checksum            16 bit
   unsigned  char ip_source[4]  ;  // IP mittente                 8 bit * 4
   unsigned  char ip_destin[4]  ;  // IP destinazione             8 bit * 4
   } ;




// definisco la struct per leggere un bit-stream con il formato del pacchetto ARP
struct pacc_arp {
  short          hw_type       ;      // short prende 2 byte  -> short <nome campo>
  short          prot_type     ;
  char           hw_size       ;
  char           prot_size     ;
  short int      op_fields     ;      // 16 bit
  char           sender_eth[6] ;      // indirizzo invio        MAC
  char           sender_ip[4]  ;      //                        IP    
  char           receiv_eth[6] ;      // indirizzo ricevente    MAC
  char           receiv_ip[4]  ;      //                        IP
  } ;


// struttura contenente indirizzi MAC - IP  (update tramite ARP)
struct arp_entry {
   unsigned char  mac[6] ;
   unsigned char  ip[4]  ;
} ;

// alloco staticamente 256 righe per la tabella
struct arp_entry arp_t[256];

int arp_n = 0; // numero di elementi presenti nella struct arp_t


// per la decisione di dove indirizzare il pacchetto ricevuto
struct decis {     //  
  unsigned int  to ;       //  indirizzo IP destinazione da inviare 
  int           ifn;       //  a che interfaccia del router deve uscire il pacchetto
 };


// definisco 2 array di indirizzi MAC e IP  da configurare con il router (indicizzati come l'interfaccia) 
unsigned char mymac[1][6] = { {0x00, 0x01, 0x02, 0x1E, 0x68, 0x37} };
unsigned char myip[1][4]  = { {147, 162, 35, 137} };

// indirizzo 147.162.35.X    ->  IP  1. interfaccia collegata a LAN ASID (host) eth0
// indirizzo  10.1.1.X       ->  IP  2. interfaccia collegata a LAN privata



// Struttura della TABELLA DI ROUTING  
// ogni pacchetto da inviare viene confontato con il contenuto di questa tabella

struct route_entry {     //  interi a 32 bit  (rapido! no confronti su vettori)
  unsigned int  dst ;    //  indirizzo destinazione
  unsigned int  msk ;    //  netmask        
  unsigned int  nxt ;    //  indirizzo destinatario  -> next hop destinazione (IP router successivo)
  unsigned int  ifn ;    //  interfaccia di uscita
 };

// dichiaro e inizializzo la tabella di routing
//                                IP dest.    netmask     nxt  ifn
//                              0. 1.  1. 10 0.255.255.255
//                              0.35.162.147   

struct route_entry r_t[2] = { { 0x0001010A,  0x00FFFFFF,   0,   3 } ,
                              { 0x0023A293,  0x00FFFFFF,   0,   2 } } ;  
int rt_n = 2;                // numero di righe in entry






/* ==================================================================================== */
unsigned short checksum(unsigned short * seq, int lung)
/* ==================================================================================== */
{
 int i;
 unsigned short oldcs, cs=0;
 for (i=0; i<lung; i++) {
   oldcs = cs;
   cs=cs+seq[i];
   if (cs < oldcs)
     cs++;
  }
  cs=0xffff-cs;
  return cs;
}



/* ==================================================================================== */
void crea_ip(struct pacc_ip pip)    // socket per invio e l'indirizzo da trovare
/* ==================================================================================== */
{
 pip.ver_len = htons(24)      ;  // versione + lung. preambolo  8 bit  
 pip.tos     = htons(0)       ;  // tipo di servizio            8 bit 
 pip.tot_len = htons(20)      ;  // lung. totale pacchetto     16 bit
 pip.identif = htons(1)       ;  // identif. per frammentaz    16 bit
 pip.ffrag   = htons(0)       ;
 pip.ttl     = htons(360)     ;  // time to live                8 bit 
 pip.prot    = htons(1)       ;  // protocollo ICMP
 pip.ip_source[0] = htons(147);
 pip.ip_source[1] = htons(162);
 pip.ip_source[2] = htons(35) ;
 pip.ip_source[3] = htons(137);

 pip.ip_destin[0] = htons(147);
 pip.ip_destin[1] = htons(162);
 pip.ip_destin[2] = htons(35);
 pip.ip_destin[3] = htons(130);

 pip.hfcs    = checksum(pip, 20) ;  // header checksum            16 bit

 return; 
}


/* ==================================================================================== */
void crea_icpm(struct pacc_icmp picmp)
/* ==================================================================================== */
{
 picmp.type   = htons(8);    
 picmp.code   = 0;
 picmp.ident  = 0;
 picmp.seqnum = 0;
 picmp.dati[0]= htons(1) ;
 picmp.dati[0]= htons(2) ;
 picmp.dati[0]= htons(3) ;
 picmp.dati[0]= htons(4) ;

 picmp.chk = checksum(picmp, 12)  ;

 return; 
}



/* ==================================================================================== */
void invia_icmp(int s0, unsigned char * buff_ip_icmp, int pktlen, struct decis d)
/* ==================================================================================== */
 {
  struct sockaddr_ll to;
  unsigned char mac[6]; 

   // data la destinazione IP, c'e' gia' nella tabella di ARP ?
  if (trova_arp(d.to, mac) == 0)   // devo riempire d.to con il MAC address 
   {
    invia_arp(s0, d);              // costruisce il pachetto a livello 2
    return;                        // invia richiesta ARP e butta via il pacchetto
   }


  // SENDTO del pacchetto IP imbustato in un frame Ethernet
  // costruisco l'Header Ethernet
  to.sll_family   = AF_PACKET;       // identifica pacchetti di livello 2 (high-level)
  to.sll_protocol = htons(ETH_P_IP); // trasporta pacchetto IP 
  to.sll_halen    = 6;               // lunghezza indirizzo campo Ethernet  MAC
  to.sll_ifindex  = 2;               // interfaccia fisica da cui inviare il pacchetto ARP 
  memcpy(to.sll_addr, mac, 6);       // indirizzo MAC

  // invio del pacchetto IP (dentro un frame Ethernet)
  if (sendto(s0, buff_ip_icmp, pktlen, 0, (struct sockaddr *) &to, sizeof(to)) == -1) {
    perror("Invio pacchetto IP + ICMP fallito -> Send non riuscita");       }
 }




/*  funzione  STAMPA_ARP   visualizza il pacchetto catturato dalla rete
    ===================    chiamata da ricevi()    */ 
void stampa_arp()
 {
  int i;
 
  for (i=0; i<arp_n; i++)            {
    printf("IP :%d.%d.%d.%d     MAC :%x:%x:%x:%x:%x:%x\n", arp_t[i].ip[0],arp_t[i].ip[1],arp_t[i].ip[2],arp_t[i].ip[3],
            arp_t[i].mac[0], arp_t[i].mac[1], arp_t[i].mac[2], arp_t[i].mac[3], arp_t[i].mac[4], arp_t[i].mac[5]);
    printf("==================       =======================\n\n"); }
 }


/*  funzione  AGGIORNA_ARP    esegue INSERT/UPDATE del MAC address nella tabella arp_t
    =====================     chiamata da ricevi()  */
int aggiorna_arp(char *mac, char *ip)
 {
  int i;

  // devo controllare se ho gia' in tabella indirizzo mac passato dalla funzione
  // scorro la tabella e controllo
  for (i=0; i< arp_n; i++) {
     // uso funzione memcmp  resitutisce 0 m se n byte sono uguali
     if (memcmp(ip,arp_t[i].ip, 4) == 0)  {  // sono 4 i byte da confrontare
        memcpy(arp_t[i].mac, mac, 6);        // default -> UPDATE dell'indirizzo MAC
        return 1;                         }  // 1 per UPDATE indirizzo MAC
                          } 

  // inserisco nuovo elemento
  memcpy(arp_t[arp_n].mac, mac, 6); 
  memcpy(arp_t[arp_n].ip, ip, 4);   
  arp_n++;                          // aggiorno arp_n

  printf("Aggiorno Num. Arp :%d",arp_n); 
  
  return 0;                         // O per INSERT nuovo indirizzo MAC
 }


/*  funzione  TROVA_ARP       trova (se presente) il valore dell'indirizzo MAC 
    ===================       chiamata da invia()    */
int trova_arp(unsigned int ip, unsigned char * mac)
 {
  int i;
 
  // ripescare il valore nella tabella arp_t.ip / arp_t.mac
  for (i=0; i< arp_n;i++)  {   // attenzione !!! IP nella tabella e' un char[4]
    if (memcmp(&ip, arp_t[i].ip, 4) == 0) {   // confronto 4 byte 1 per 1
       memcpy(mac, arp_t[i].mac, 6);          // copio i 6 valori dell'indirizzo
       return 1;                              // ok! trovato
      }
    }
  return 0;                                   // non trovato e ritorno 
 }


/*  funzione  INVIA_ARP    invia la richiesta di ARP (indirizzo MAC non trovato !)
    ===================    chiamata da invia()  */
void invia_arp(int s0, struct decis d)    // socket per invio e l'indirizzo da trovare
 {
  struct sockaddr_ll to;
  struct pacc_arp ap;

  // SEND con pacchetto Ethernet  Header Ethernet
  to.sll_family   = AF_PACKET;         // identifica pacchetti di livello 2 (high-level)
  to.sll_protocol = htons(ETH_P_ARP);  // protocollo dentro il frame (e' ARP)   
  to.sll_ifindex  = htons(2);          // interfaccia fisica da cui inviare il pacchetto ARP 

  // pacchetto in broadcast  -> tutti i bit a 1 
  to.sll_addr[0]  = 0xFF;             // 6 byte a 255  0xFF   indirizzo MAC
  to.sll_addr[1]  = 0xFF;             // 6 byte a 255  0xFF   indirizzo MAC
  to.sll_addr[2]  = 0xFF;             // 6 byte a 255  0xFF   indirizzo MAC
  to.sll_addr[3]  = 0xFF;             // 6 byte a 255  0xFF   indirizzo MAC
  to.sll_addr[4]  = 0xFF;             // 6 byte a 255  0xFF   indirizzo MAC
  to.sll_addr[5]  = 0xFF;             // 6 byte a 255  0xFF   indirizzo MAC
  // to.sll_halen    = 6;                // ??? lunghezza indirizzo  campo Ethernet  MAC

  // devo costruire il pacchetto ARP da mandare in rete
  ap.hw_type      = htons(1);      // short prende 2 byte  -> short <nome campo>
  ap.prot_type    = htons(0x0800); // contiene pacchetto IP
  ap.hw_size      = 6 ;
  ap.prot_size    = 4 ;
  ap.op_fields    = htons(1) ;
  
  // inizializzo i valori MAC e IP source a seconda dell'interfaccia da dove spedire frame
  memcpy(ap.sender_eth, mymac[d.ifn-2],6);  // indirizzo MAC nostro router
  memcpy(ap.sender_ip, myip[d.ifn-2],4);    // indirizzo IP  nostro router

  ap.receiv_eth[0] = 0;      // indirizzo MAC destinazione (non conosciuto !!)
  ap.receiv_eth[1] = 0;
  ap.receiv_eth[2] = 0;
  ap.receiv_eth[3] = 0;
  ap.receiv_eth[4] = 0;
  ap.receiv_eth[5] = 0;
  memcpy(ap.receiv_ip, &d.to, 4) ;  // indirizzo IP destinazione di cui non conosciamo il MAC

  printf("Invia ARP -> struct decis d.to %d:    d.ifn :%d:\n", d.to ,d.ifn);
  //printf("Invia ARP -> struct decis d.to %d.%d.%d.%d:    d.ifn :%d:\n", (char *)&(d.to)[0] ,
  //     (char *)&(d.to)[1] ,(char *)&(d.to)[1] ,(char *)&(d.to)[1] ,d.ifn);
  //scanf("%s",&cc);    

  // invio il pacchetto ARP  (dentro un frame Ethernet)
  if (sendto(s0,(unsigned char *) &ap, sizeof(ap), 0, (struct sockaddr *) &to, sizeof(to)) == -1){
     perror("Invio pacchetto Ethernet + ARP fallito -> SENDTO non riuscita"); }
 }


/*  funzione  RICEVI       riceve dalla rete un bit-stream 
    ================       e lo struttura come pacchetto   */
int ricevi(int s, char * buffer, int * lung_pacc)
 {
  int len, from_len, i; 
  struct pacc_arp *arp_pack;
  char cc;

  // ciclo per cattura solo pacchetti definiti A.R.P.  in frame type '0x0806'
  // devo caricare il contenuto della struttura 
  from.sll_family   = AF_PACKET;         // identifica pacchetti di livello 2 (high-level)
  from.sll_protocol = htons(ETH_P_ALL);  // trasporta pacchetto ARP 
  from.sll_ifindex  = 0;
  from.sll_hatype   = 0;                 // serve per la SEND invio (tipo ARP)
  from.sll_pkttype  = 0;                 // anche PACKET_BROADCAST !!
  // from.sll_halen  = 0;  // filtro indirizzo MAC da cui vengono i pacchetti  
  // from.sll_addr[8]=     // indirizzo MAC di 48 bit 

  // lunghezza della struttura -> serve alla recvfrom
  from_len = sizeof(struct sockaddr_ll);

  // la RECVFROM sta in attesa fintantoche' non arriva un pacchetto dalla rete 
  // inserisco i dati del frame trovato in buffer , ritorno il n. byte letti
  len = recvfrom(s, buffer, ETH_SIZE, 0, (struct sockaddr*) &from, &from_len);
  *lung_pacc = len;

  if (len == -1) {
     perror("RECEIVE FROM fallita");
     return 1;   }

  // controllo il tipo di pacchetto arrivato attraverso la recvfrom

  //  se e' un pacchetto ARP (codice 0608)
  if (from.sll_protocol == htons(0x0806))  {  // senza htons() 0x0608 
     arp_pack = (struct pacc_arp *) buffer ;

     // controllo presenza indirizzi nella arp_entry passando  MAC address e IP
     if (aggiorna_arp( (*arp_pack).sender_eth, (*arp_pack).sender_ip ) == 0) {
       printf("Aggiorna ARP"); 
       //scanf("%s",&cc);
       stampa_arp();   }         

     return 1;                            } 

  // controllo se ho un pacchetto IP   campo Type del pacchetto  
  if (from.sll_protocol == htons(0x0800))  {  // senza htons() 0x0008 
    /*printf("ho ricevuto pacchetto IP %x\n",htons(from.sll_protocol)); */


    /*  devo controllare  il  pacchetto  ICMP  */
    if (buffer[10] == 1) {




    }


      




    return 0;                             
    }

  return 1;   // chiudo la funzione ricevi
 }


/*  funzione  DECIDI        
    ================ */
int decidi(unsigned char * buffer, struct decis *d)
 {
  int i, trovato = 0;
  unsigned int *dst;                      // puntatore alla destinazione
  dst = (unsigned int *) &buffer[16];     // 16.a posizione dell'array buffer  (IP destinazione)
  // dst = (unsigned int *) (buffer+16);     sono equivalenti
  // dst = ((unsigned int *) buffer) + 4;    sono long di 4 -> 4 * 4

  // controllo contenuto dell'array  
  for (i=0; i < rt_n && ! trovato ; i++)
   {
     //  funzione fondamentale: prendi indirizzo IP destinazione del pacchetto (long int)
     //                         mascheralo tramite netmask (definita in tabella routing)
     //                         se il confronto (& bit a bit) e' positivo
     //                            ho trovato la riga con le informazioni cercate
     //                         altrimenti
     //                            non conosco percorso per IP destinazione
     //                            e butto via il pacchetto

     if ( (*dst & r_t[i].msk) == (r_t[i].dst & r_t[i].msk) )
      { 
       // ho trovato il math (collegamento) tra IP pacchetto da inviare e riga della
       // tabella di routing  
       trovato = 1;            // return della funzione per invia() o meno                                   

       // problema -> decidere l'interfaccia di uscita  LAN ASID->0 / LAN esterna->1
       // caso LAN in cui presente il router --->  r_t_nxt 0.0.0.0 
       //                                          prossimo hop  la destinazione
       if (r_t[i].nxt == 0)  
         (*d).to = *dst;         //  [d->to  == (*d).to]  invio alla destinazione
       else
         (*d).to = r_t[i].nxt;   // invia al next hop (altro router)
        
       (*d).ifn = r_t[i].ifn;    // l'interfaccia del router dove uscire 
      }  // chiudo la if
   }     // chiudo il for

  if (trovato)   
   {
    printf("Ho deciso per : %d.%d.%d.%d\n", *((unsigned char *) dst),
          *((unsigned char *) dst+1), *((unsigned char *) dst+2),
          *((unsigned char *) dst+3));

    //printf("Interfaccia -> %d",d->ifn);
    //scanf("%s",&cc);
   }  

  if (!trovato)
   {
    printf("Non so come raggiungere : %d.%d.%d.%d\n", *((unsigned char *) dst),\
          *((unsigned char *) dst+1), *((unsigned char *) dst+2),              \
          *((unsigned char *) dst+3));
   }  
  
  return trovato;       // valore di controllo per operazioni di invia()
 }




/*  funzione  INVIA     
    ================ */
void invia(int s0, unsigned char * buffer, int pktlen, struct decis d)
 {
 // 1) ho prelevato il pacchetto dalla rete
 // 2) e' un pacchetto IP da inoltrare
 // 3) ho trovato il match (corrispondenza) nella tabella di routing
 // 4) cerco l'indirizzo MAC corrispondente all'IP destinazione (tabella arp_entry)
 // 5) se non trovo la riga corrispondente
 //       uso il pacchetto per invio di una richiesta ARP e butto via il pacchetto
 //    altrimenti
 //       costruisco un frame Ethernet (tipo pacchetto IP) e lo invio 
 
 // destinazione e interfaccia  presenti nella struttura  -> to
 // se un pacchetto non trova corrispondenza nella tabella scateno ARP e butto via pacchetto

 struct sockaddr_ll to;
 unsigned char mac[6]; 

 // data la destinazione IP, c'e' gia' nella tabella di ARP ?
 if (trova_arp(d.to, mac) == 0)   // devo riempire d.to con il MAC address 
  {
   invia_arp(s0, d);              // costruisce il pachetto a livello 2
   return;                        // invia richiesta ARP e butta via il pacchetto
  }

  // SENDTO del pacchetto IP imbustato in un frame Ethernet
  //    costruisco l'Header Ethernet
  to.sll_family   = AF_PACKET;       // identifica pacchetti di livello 2 (high-level)
  to.sll_protocol = htons(ETH_P_IP); // trasporta pacchetto IP 
  to.sll_halen    = 6;               // lunghezza indirizzo campo Ethernet  MAC
  to.sll_ifindex  = d.ifn;           // interfaccia fisica da cui inviare il pacchetto ARP 
  memcpy(to.sll_addr, mac, 6);       // indirizzo MAC

  printf("MAC : %s",mac);
  printf("\n==========================\n");
  //scanf("%s",&cc);    

  // invio del pacchetto IP (dentro un frame Ethernet)
  if (sendto(s0, buffer, pktlen, 0, (struct sockaddr *) &to, sizeof(to)) == -1) {
    perror("Invio pacchetto Ethernet + IP fallito -> Send non riuscita");       }
 }


/*  MAIN  PROGRAM 
   ==============  */
int main()
 {
  unsigned char buffer[1500] ;    // contiene frame Ethernet  (puntatore)
  int s0;                         // descrittore del socket
  int n;
  int pkt_len;                    // lunghezza del pacchetto
  struct decis decisione;         // scelta dell'interfaccia di output


  //unsigned char * p_ip;           // puntatore alla struct IP   
  unsigned char p_ip[20];
  unsigned char p_icmp[12];       // array contenente pacchetto ICMP   
  unsigned char buff_ip_icmp[32] ;    // contiene frame Ethernet  (puntatore)




  // connessione  -> da eseguire una sola volta
  s0 = socket(PF_PACKET, SOCK_DGRAM, htons(ETH_P_ALL));
  // controllo il return value del socket
  if (s0 == -1) {
     perror("SOCKET fallita");
     return 1;  }


  // invio il pacchetto all'indirizzo prestabilito
  decisione.to = inet_addr("147.162.35.130");
  decisione.ifn = 2;

  invia_arp(s0, decisione);              // costruisce il pachetto a livello 2

  crea_ip(p_ip);                         // inizializzo pacchetto IP
  crea_icmp(p_icmp);                     // inizializzo pacchetto ICMP 
  // creo il pacchetto da inviare 
  memcpy(buff_ip_icmp,p_ip,20);
  memcpy(buff_ip_icmp,p_icmp,12);


  while (1)
   {     // ciclo infinito
     
    // ricevo frame a livello 2 
    n = ricevi(s0, buffer, &pkt_len) ;  // attendo arrivo del pacchetto  -> recvfrom()

    // manca la bufferizzazione del pacchetto che non so dove inviare
    // in attesa del ritorno dell'indirizzo MAC dovuto ad un precedente invio di ARP

    // il pacchetto interessa al routing -> pacchetto IP
    if (n == 0)
     {
       // controllo contenuto indirizzo IP a livello 3 
       if (decidi(buffer, &decisione) == 1)
                                                  
       // creo frame Ethernet per inviare il pacchetto ricevuto (forwarding  livello 2)
       //invia(s0, buffer, pkt_len, decisione) ;  

       invia_icpm(s0, buff_ip_icmp, pkt_len) ;  

     }   
   }  // chiudo il while
 }














