/* -------------------------------------------------------------------------- */
/* ! PRJ: arp - arp.c - file principale ------------------------------------- */
/* -------------------------------------------------------------------------- */


/* -------------------------------------------------------------------------- *
 * Headers delle funzioni da usare                                            *
 * -------------------------------------------------------------------------- */
#include <sys/socket.h>             /* funzioni per la creazione di sockets   */
#include <linux/if_arp.h>           /* defines, structs per il protocollo ARP */
#include <stdlib.h>                 /* exit(), abort(), ecc...                */
#include <stdio.h>                  /* standard I/O streams                   */
#include <features.h>               /* controllo versione librerie C/C++      */
#if __GLIBC__ >= 2 && __GLIBC_MINOR >= 1 /* se la versione e' >= alla 2.1     */
    #include <netpacket/packet.h>	/* strutture per i pacchetti, defines     */
    #include <net/ethernet.h>		/* headers addizionali, ecc...			  */
#else								/* la versione delle Glibc e' < alla 2.1  */
    #include <asm/types.h>			/* tipi di variabili addizionali          */
    #include <linux/if_packet.h>	/* strutture per i pacchetti, defines     */
    #include <linux/if_ether.h>		/* strutture per i pacchetti, defines     */
#endif


/* -------------------------------------------------------------------------- *
 * Defines                                                                    *
 * -------------------------------------------------------------------------- */
#define L_BUFF     1500             /* dimensione massima del buffer usato    */


/* -------------------------------------------------------------------------- *
 * Strutture                                                                  *
 * -------------------------------------------------------------------------- */
struct arp_packet {                 /* record del pacchetto arp               */
    unsigned short hwtype;          /* tipo della tecnologia di rete          */
    unsigned short prottype;        /* tipo di protocollo                     */
    unsigned char  hwsize;          /* dimensione dell'indirizzo (Ethernet=6) */
    unsigned char  protsize;        /* dimensione del protocollo              */
    unsigned short op;              /* operazione da farsi                    */
    unsigned char  sender_ha[6];    /* indirizzo MAC mittente                 */
    unsigned char  sender_pa[4];    /* indirizzo IP mittente                  */
    unsigned char  receiver_ha[6];  /* indirizzo MAC destinatario             */
    unsigned char  receiver_pa[4];  /* indirizzo IP destinatario              */
};/* struct arp_packet */


/* -------------------------------------------------------------------------- *
 * Prototipi delle funzioni usate                                             *
 * -------------------------------------------------------------------------- */
void ifexit (int cond,int exit_code,char msg[]);


/* -------------------------------------------------------------------------- *
 * MAIN: funzione principale                                                  *
 * -------------------------------------------------------------------------- *
 * I : \   | \                                                                *
 * O : int | codice di uscita dal programma                                   *
 * -------------------------------------------------------------------------- */
int main () {
	unsigned char buff[L_BUFF];		/* buffer per la ricezione dei pacchetti  */
	struct sockaddr_ll from;		/* link layer struct sockaddr             */
	struct arp_packet *arp_pt;		/* puntatore al pacchetto arp			  */
    int s,                          /* identificativo dell'handler del socket */
		i,                          /* variabile contatore                    */
		len,                        /* lunghezza del pacchetto ricevuto       */
		from_len;					/* dimensione della sockaddr_ll usata     */

    /* apertura del socket per le ricezioni                                   */
	s=socket(PF_PACKET,SOCK_DGRAM,htons(ETH_P_ALL));
    ifexit(s==-1,-1,"Inizializzazione socket fallita");

	/* lettura pacchetti finche' non viene incontrato un ARP                  */
	while (from.sll_protocol!=htons(ETH_P_ARP)) {
	    /* inizializzazione pacchetto per il passaggio alla "recvfrom"        */
		from.sll_family=AF_PACKET;
		from.sll_protocol=htons(ETH_P_ALL);
		from.sll_ifindex=0;
		from.sll_hatype=ARPHRD_ETHER;
		from.sll_pkttype=PACKET_HOST;
		from.sll_halen=0;

		from_len=sizeof(struct sockaddr_ll); /* dimensione della sockaddr_ll  */

	    /* ricezione del pacchetto                                            */
		len=recvfrom(s,buff,L_BUFF,0,(struct sockaddr *)&from,&from_len);
		ifexit(len==-1,-2,"recvfrom fallita");
    }/* while */

    /* stampa del contenuto del pacchetto                                     */
    for(i=0;i<len;i++) {
		if (i%4==0)
			printf("\n\t");
		printf("%x(%d)\t",buff[i],buff[i]);
    }/* for */

	/* stampa di alcune informazioni all'interno del pacchetto                */
    arp_pt=(struct arp_packet *)buff;
    printf("\n\n\tTipo operazione..: %d\n",htons(arp_pt->op));
    printf("\tIP sorgente......: %d.%d.%d.%d\n",
		   arp_pt->sender_pa[0],arp_pt->sender_pa[1],
		   arp_pt->sender_pa[2],arp_pt->sender_pa[3]);
    printf("\tIP destinazione..: %d.%d.%d.%d\n\n",
		   arp_pt->receiver_pa[0],arp_pt->receiver_pa[1],
		   arp_pt->receiver_pa[2],arp_pt->receiver_pa[3]);

	/* chiusura della connessione       									  */
	shutdown(s,SHUT_RDWR);
	close(s);

	return 0;						/* programma terimnato correttamente	  */
}/* main */


/* -------------------------------------------------------------------------- *
 * IFEXIT: esce se la condizione passata e' vera                              *
 * -------------------------------------------------------------------------- *
 * I1: cond      | risultato della condizione passata (0:falso, 1:vero)       *
 * I2: exit_code | codice d'uscita da ritornare                               *
 * I3: msg       | messaggio da stampare                                      *
 * O : \         | \                                                          *
 * -------------------------------------------------------------------------- */
void ifexit (int cond,int exit_code,char msg[]) {
    if (cond) {                     /* se la condizione e' vera               */
        perror(msg);                /* stampa del messaggio d'errore          */
        exit(exit_code);            /* uscita dal programma                   */
    }/* if */
}/* ifexit */

