/* -------------------------------------------------------------------------- */
/* ! PRJ: browser - navigator.c - file principale --------------------------- */
/* -------------------------------------------------------------------------- */


/* -------------------------------------------------------------------------- *
 * Headers delle funzioni da usare                                            *
 * -------------------------------------------------------------------------- */
#include <stdio.h>                  /* gestione dell'I/O                      */
#include <sys/socket.h>             /* gestione delle connessioni socket      */
#include <unistd.h>                 /* gestione dei file descriptor           */
#include <netinet/in.h>             /* gestione indirizzi internet            */
#include <string.h>                 /* gestione delle stringhe                */


/* -------------------------------------------------------------------------- *
 * Strutture                                                                  *
 * -------------------------------------------------------------------------- */
struct sockaddr_in ind;             /* struttura per il socket del server     */


/* -------------------------------------------------------------------------- *
 * Defines                                                                    *
 * -------------------------------------------------------------------------- */
#define LRISP 60000                 /* lunghezza massima della risposta       */
/* macro per convertire l'indirizzo IP in unsigned int dalla forma a.b.c.d    */
#define getIP(a,b,c,d) htonl( (unsigned long)a*0x1000000+b*0x10000+c*0x100+d )


/* -------------------------------------------------------------------------- *
 * Prototipi delle funzioni usate                                             *
 * -------------------------------------------------------------------------- */
void ifexit (int cond,int exit_code,char msg[]);


/* -------------------------------------------------------------------------- *
 * MAIN: funzione principale                                                  *
 * -------------------------------------------------------------------------- *
 * I : \   | \                                                                *
 * O : int | codice di uscita dal programma                                   *
 * -------------------------------------------------------------------------- */
int main() {
    char req[]="GET /index.htm\n",      /* richiesta del client               */
         risp[LRISP];                   /* risposta del server                */
    int  s,                             /* handle del socket                  */
         n,                             /* n char letti                       */
         tmp;                           /* variabile temporanea               */

    /* apertura del socket per le ricezioni                                   */
    s=socket(PF_INET,SOCK_STREAM,0);
    ifexit(s==-1,-1,"Inizializzazione socket fallita");

    /* inizializzazione socket                                                */
    ind.sin_family=AF_INET;
    ind.sin_port=htons(80);                 /* porta                          */
    ind.sin_addr.s_addr=getIP(127,0,0,1);   /* indirizzo                      */

    /* connessione                                                            */
    tmp=connect(s,(struct sockaddr *)&ind,sizeof(struct sockaddr_in));
    ifexit(tmp!=0,-2,"Connessione fallita");

    /* invio della richiesta                                                  */
    tmp=write(s,req,strlen(req));
    ifexit(tmp==-1,-3,"Richiesta fallita");

    /* lettura e stampa della risposta                                        */
    while(n=read(s,risp,LRISP-1)) {
        ifexit(n==-1,-4,"Lettura fallita");
        risp[n]='\0';
        printf("%s\n",risp);
    }/* while */

	/* chiusura della connessione       									  */
	shutdown(s,SHUT_RDWR);
    close(s);

    return 0;                       /* programma terminato correttamente      */
}/* main */


/* -------------------------------------------------------------------------- *
 * IFEXIT: esce se la condizione passata e' vera                              *
 * -------------------------------------------------------------------------- *
 * I1: cond      | risultato della condizione passata (0:falso, 1:vero)       *
 * I2: exit_code | codice d'uscita da ritornare                               *
 * I3: msg       | messaggio da stampare                                      *
 * O : \         | \                                                          *
 * -------------------------------------------------------------------------- */
void ifexit (int cond,int exit_code,char msg[]) {
    if (cond) {                     /* se la condizione e' vera               */
        perror(msg);                /* stampa del messaggio d'errore          */
        exit(exit_code);            /* uscita dal programma                   */
    }/* if */
}/* ifexit */
