/* -------------------------------------------------------------------------- */
/* ! PRJ: proxy - proxy.c - file principale --------------------------------- */
/* -------------------------------------------------------------------------- */


/* -------------------------------------------------------------------------- *
 * Headers delle funzioni da usare                                            *
 * -------------------------------------------------------------------------- */
#include <stdio.h>                  /* gestione dell'I/O                      */
#include <sys/socket.h>             /* gestione delle connessioni socket      */
#include <unistd.h>                 /* gestione dei file descriptor           */
#include <netinet/in.h>             /* gestione indirizzi internet            */
#include <string.h>                 /* gestione delle stringhe                */
#include <sys/types.h>				/* gestione di altri tipi di dati		  */
#include <netdb.h>					/* funzioni per l'ARP ecc...			  */


/* -------------------------------------------------------------------------- *
 * Defines                                                                    *
 * -------------------------------------------------------------------------- */
#define LRISP 60000                 /* lunghezza massima della risposta       */
/* macro per convertire l'indirizzo IP in unsigned int dalla forma a.b.c.d    */
#define getIP(a,b,c,d) htonl( (unsigned long)a*0x1000000+b*0x10000+c*0x100+d )


/* -------------------------------------------------------------------------- *
 * Strutture                                                                  *
 * -------------------------------------------------------------------------- */
struct sockaddr_in ind_px,			/* struttura per il socket del proxy      */
				   ind_cl,			/* struttura per il socket del client     */
				   ind_sv;			/* struttura per il socket del server     */


/* -------------------------------------------------------------------------- *
 * Prototipi delle funzioni usate                                             *
 * -------------------------------------------------------------------------- */
void ifexit (int cond,int exit_code,char msg[]);


/* -------------------------------------------------------------------------- *
 * MAIN: funzione principale                                                  *
 * -------------------------------------------------------------------------- *
 * I : \   | \                                                                *
 * O : int | codice di uscita dal programma                                   *
 * -------------------------------------------------------------------------- */
int main() {
	int  s_px,						/* handle del socket del proxy			  */
		 s_cl,						/* handle del socket del client			  */
		 s_sv,						/* handle del socket del server			  */
	     n,							/* n di bytes letti                       */
	     ind_cl_size,				/* dimensione della struttura             */
	     i,							/* var contatore                          */
		 tmp;						/* var temporanea                         */
	char risp[LRISP],				/* buffer della risposta del server       */
	     ask[LRISP],				/* richiesta da passare al server		  */
	     nome_server[200],			/* nome del server						  */
	     *file_name;				/* puntatore al nome del file richiesto   */
	FILE *pf;						/* puntatore al file da spedire			  */
	struct hostent *server_he;		/* pt alla struttura hostent del server   */

    /* apertura del socket per le ricezioni                                   */
    s_px=socket(PF_INET,SOCK_STREAM,0);
    ifexit(s_px==-1,-1,"Inizializzazione socket proxy fallita");

    /* inizializzazione socket                                                */
	ind_px.sin_family=AF_INET;
	ind_px.sin_port=htons(8700);    /* porta dove attendere le richieste      */

    /* binding                                                                */
    tmp=bind(s_px,(struct sockaddr *)&ind_px,sizeof(struct sockaddr_in));
    ifexit(tmp!=0,-2,"Bind fallita");

    /* inizio attesa di connessioni                                           */
    tmp=listen(s_px,SOMAXCONN);
    ifexit(tmp==-1,-3,"Listen fallita");

    while (1) {                     /* attesa di richieste all'infinito       */
        /* lettura del socket di un client in coda                            */
        s_cl=accept(s_px,(struct sockaddr *)&ind_cl,&ind_cl_size);
        ifexit(s_cl==-1,-4,"Accept fallita");

        /* lettura della richiesta                                            */
		n=read(s_cl,risp,LRISP-1);
        ifexit(n==-1,-5,"Lettura fallita");
		risp[n]='\0';
		printf("\n-----\t-----\t-----\t-----\t-----\t-----\t-----\n%s",risp);

		/* estrazione del nome del server									  */
		for (i=11;risp[i]!='/';i++)
			nome_server[i-11]=risp[i];
		nome_server[i-11]='\0';
		printf("Il nome del server e': %s\n",nome_server);

		/* estrazione del nome del nome del file							  */
		file_name=&risp[i+1];
		for (i=0;file_name[i]!=' ';i++) 
			/* NOP */;
		file_name[i]='\0';
		printf("Il nome del file e': %s\n",file_name);

		/* risoluzione dell'indirizzo del server							  */
		server_he=gethostbyname(nome_server);
		printf("Indirizzo del server: %d.%d.%d.%d\n",
				(unsigned char)(server_he->h_addr[0]),
				(unsigned char)(server_he->h_addr[1]),
				(unsigned char)(server_he->h_addr[2]),
				(unsigned char)(server_he->h_addr[3])
			   );

        /* socket per la connessione al server                                */
		s_sv=socket(PF_INET,SOCK_STREAM,0);
	    ifexit(s_sv==-1,0,"Inizializzazione socket server fallita");

	    /* inizializzazione socket                                            */
		ind_sv.sin_family=AF_INET;
		ind_sv.sin_port=htons(80);
		memcpy(&ind_sv.sin_addr.s_addr,server_he->h_addr,4);

	    /* connessione al server                                              */
		tmp=connect(s_sv,(struct sockaddr *)&ind_sv,sizeof(struct sockaddr_in));
		if (tmp!=0) {
			perror("Connessione al server fallita");
			sprintf(risp,"<html><body>Richiesta fallita!</body></html>");
			tmp=write(s_cl,risp,strlen(risp));
			ifexit(tmp<0,-9,"Connessione al server fallita");
			continue;
		}/* if */

	    /* invio della richiesta al server                                    */
		sprintf(ask,"GET /%s\n\n",file_name);
		tmp=write(s_sv,ask,strlen(ask));
		if (tmp<0) {
			shutdown(s_sv,SHUT_RDWR);
			close(s_sv);
			perror("Richiesta al server fallita");
			sprintf(risp,"<html><body>Richiesta fallita!</body></html>");
			tmp=write(s_cl,risp,strlen(risp));
			ifexit(tmp<0,-10,"Richiesta al server fallita");
			continue;
		}/* if */

	    /* lettura/invio della risposta                                       */
		while(n=read(s_sv,risp,LRISP-1)) {
			if (n==-1) {
				perror("Lettura fallita");
				break;
			}/* if */
			risp[n]='\0';
			tmp=write(s_cl,risp,n);
			ifexit(tmp<0,-11,"Risposta fallita");
		}/* while */


	    /* chiusura delle connessioni                                         */
		shutdown(s_sv,SHUT_RDWR);
		shutdown(s_cl,SHUT_RDWR);
		close(s_sv);
		close(s_cl);
	}/* while */

	/* chiusura della connessione proxy 									  */
	shutdown(s_px,SHUT_RDWR);
	close(s_px);

    return 0;                       /* programma terminato correttamente      */
}/* main */


/* -------------------------------------------------------------------------- *
 * IFEXIT: esce se la condizione passata e' vera                              *
 * -------------------------------------------------------------------------- *
 * I1: cond      | risultato della condizione passata (0:falso, 1:vero)       *
 * I2: exit_code | codice d'uscita da ritornare                               *
 * I3: msg       | messaggio da stampare                                      *
 * O : \         | \                                                          *
 * -------------------------------------------------------------------------- */
void ifexit (int cond,int exit_code,char msg[]) {
	if (exit_code==0&&cond) {		/* se exit_code e' = a 0 non uscire       */
		perror(msg);
	}else if (cond) {               /* se la condizione e' vera               */
        perror(msg);                /* stampa del messaggio d'errore          */
        exit(exit_code);            /* uscita dal programma                   */
    }/* if */
}/* ifexit */
