/* -------------------------------------------------------------------------- */
/* ! PRJ: router - arp_table.c - gestione tabella corrispondenze MAC-IP ----- */
/* -------------------------------------------------------------------------- */


#include "router.h"                 /* file header del progetto               */


/* -------------------------------------------------------------------------- *
 * Variabili globali                                                          *
 * -------------------------------------------------------------------------- */
struct arp_entry at[L_AT];          /* tabella per le corrispondenze MAC-IP   */
int    at_n=0;                      /* dimensione della tabella ARP           */


/* -------------------------------------------------------------------------- *
 * INSERT_ARP: inserisce un record nella tabella per le corrispondenze MAC-IP *
 * -------------------------------------------------------------------------- *
 * I1: ip  | indirizzo IP                                                     *
 * I2: mac | indirizzo MAC                                                    *
 * O : \   | \                                                                *
 * -------------------------------------------------------------------------- */
void insert_arp (unsigned char *ip,unsigned char *mac) {
    int i;                          /* variabile contatore                    */
    for (i=0;i<4;i++)               /* copiatura dell'indirizzo IP            */
        at[at_n].ip[i]=ip[i];
    for (i=0;i<6;i++)               /* copiatura dell'indirizzo MAC           */
        at[at_n].mac[i]=mac[i];
    at_n++;                         /* incremento del numero di record        */
}/* insert_arp */


/* -------------------------------------------------------------------------- *
 * CERCA_MAC: cerca un record nella tabella per le corrispondenze MAC-IP      *
 * -------------------------------------------------------------------------- *
 * I1: ip  | indirizzo IP                                                     *
 * I2: mac | indirizzo MAC                                                    *
 * O : int | 1=trovato, 0=non trovato                                         *
 * -------------------------------------------------------------------------- */
int cerca_mac (unsigned char *ip,unsigned char *mac) {
    int i,j;                        /* variabili contatore                    */
    for (i=0;i<at_n;i++)            /* per ogni record, se gli IP coincidono  */
        if (*((unsigned long *)ip)==*((unsigned long *)(at[i].ip))) {
            for (j=0;j<6;j++)		/* copiatura dell'indirizzo MAC           */
                mac[j]=at[i].mac[j];
            return 1;               /* ritorno esito positivo                 */
        }/* if */
    return 0;                       /* ritorno esito negativo                 */
}/* cerca_mac */


/* -------------------------------------------------------------------------- *
 * STAMPA_AT: stampa la tabella per le corrispondenze MAC-IP                  *
 * -------------------------------------------------------------------------- *
 * I : \ | \                                                                  *
 * O : \ | \                                                                  *
 * -------------------------------------------------------------------------- */
void stampa_at () {
    int i;                          /* variabile contatore                    */
    printf("\n-------------------------------------------------------------\n");
    for (i=0;i<at_n;i++) {
        printf("%3d) ",i);          /* stampa dell'indice del record          */
        printf("%3d.%3d.%3d.%3d    <-->    %2x : %2x : %2x : %2x : %2x : %2x\n",
               at[i].ip[0],at[i].ip[1],at[i].ip[2],at[i].ip[3], /* IP addr.   */
               at[i].mac[0],at[i].mac[1],at[i].mac[2],          /* MAC addr.  */
               at[i].mac[3],at[i].mac[4],at[i].mac[5]);
    }/* for */
    printf("-------------------------------------------------------------\n");
}/* stampa_at */

