/* -------------------------------------------------------------------------- */
/* ! PRJ: router - route_table.c - gestione tabella di routing -------------- */
/* -------------------------------------------------------------------------- */


#include "router.h"                 /* file header del progetto               */


/* -------------------------------------------------------------------------- *
 * Variabili globali                                                          *
 * -------------------------------------------------------------------------- */
struct route_entry rt[L_TR];        /* tabella di routing                     */
int rt_n;                           /* dimensione della tabella di routing    */


/* -------------------------------------------------------------------------- *
 * INIT_RT: iniziallizza la tabella di routing                                *
 * -------------------------------------------------------------------------- *
 * I : \ | \                                                                  *
 * O : \ | \                                                                  *
 * -------------------------------------------------------------------------- */
void init_rt () {
    /* record #0 - il pacchetto appartine alla rete attuale                   */
    rt[0].dest=getIP(147,162,35,0); /* IP: 147.162.35.0          (ASID)       */
    rt[0].mask=htonl(~0xFF);		/* maschera 255.255.255.0                 */
    rt[0].nextip=0;                 /* quello che sta scritto nel pacchetto   */
    rt[0].ifn=2;                    /* interfaccia numero 2                   */

    /* record #1 - bisogna spedire il pacchetto al router dell'altra rete     */
    rt[1].dest=htonl(0xA010100);    /* IP: 10.1.1.0              (laptop)     */
    rt[1].mask=htonl(~0xFF);		/* maschera 255.255.255.0                 */
    rt[1].nextip=0;                 /* quello che sta scritto nel pacchetto   */
    rt[1].ifn=3;                    /* interfaccia numero 3                   */

    /* record #1 - default -> internet                                        */
    rt[2].dest=0;                   /* destinazione 0.0.0.0                   */
    rt[2].mask=0;                   /* maschera 0.0.0.0                       */
    rt[2].nextip=getIP(147,162,35,254); /* nextIP: 147.162.35.254             */
    rt[2].ifn=2;                    /* interfaccia numero 2                   */

    rt_n=3;                         /* aggiornamento del numero di record     */
}/* init_route_table */


/* -------------------------------------------------------------------------- *
 * DECIDI:                                                                    *
 * -------------------------------------------------------------------------- *
 * I1: buff   | pt al pacchetto da analizzare                                 *
 * I2: ifn    | pt al numero di interfaccia da usare                          *
 * I3: nextip | pt all'indirizzo IP del prossimo hop                          *
 * O : int    | 0=IP pari a 0.0.0.0, 1=coppia IP-MAC restituita               *
 * -------------------------------------------------------------------------- */
int decidi (unsigned char *buff,int *ifn,unsigned int *nextip) {
    unsigned int *dest;				/* pt all'indirizzo IP di destinazione    */
    int trovato=0,					/* var spia: 1=trovato, 0=non trovato     */
		i;							/* variabile contatore                    */

    dest=((unsigned int *)buff)+4;	/* dest punta all'indirizzo IP nel buffer */
    for(i=0;i<rt_n&&trovato==0;i++) { /* scansione della tabella di routing   */
        if(*dest&rt[i].mask==rt[i].dest&rt[i].mask) { /* se avviene un match  */
            trovato=1;				/* indirizzo IP trovato nella tabella     */
            *ifn=rt[i].ifn;			/* copiatura del numero di interfaccia    */
			if (rt[i].nextip)		/* se l'IP per il prossimo hop non e' 0   */
				*nextip=rt[i].nextip;  /* assegna l'IP letto dalla tabella    */
			else					/* l'IP per il prossimo hop non e' 0      */
				*nextip=*dest;		/* assegna l'IP letto dal buffer          */
        }/* if */
    }/* for */
    if (trovato==0)					/* se l'ind. IP non e' stato trovato      */
        return 1;	/*!! errore !!*/
    return 0;
}/* decidi */

