/* -------------------------------------------------------------------------- */
/* ! PRJ: router - router.c - file principale ------------------------------- */
/* -------------------------------------------------------------------------- */


#include "router.h"                 /* file header del progetto               */


/* -------------------------------------------------------------------------- *
 * MAIN: funzione principale                                                  *
 * -------------------------------------------------------------------------- *
 * I : \   | \                                                                *
 * O : int | codice di uscita dal programma                                   *
 * -------------------------------------------------------------------------- */
int main () {
    unsigned short prot;            /* tipo di protocollo                     */
    unsigned int   nextip;          /* indirizzo IP del prossimo hop          */
    unsigned char  nextmac[6],      /* indirizzo mac del prossimo hop         */
                   *ip,             /* pt a indirizzo IP                      */
                   *mac;            /* pt a indirizzo MAC                     */
    char buff[L_BUFF];              /* buffer per la lettura dei pacchetti    */
    int  l,                         /* numero di bytes contenuti nel buffer   */
         s,                         /* identificativo dell'handler del socket */
         interface;                 /* interfaccia di destinazione            */

    init_rt();                      /* inizializzazione tabella di routing    */

    /* apertura del socket per le ricezioni:                                  */
    /*  PF_PACKET  > offers a low level packet interface                      */
    /*  SOCK_DGRAM > connectionless,unreliable messages of a fixed max length */
    /*  ETH_P_ALL  > every packet (protocol type)                             */
    s=socket(PF_PACKET,SOCK_DGRAM,htons(ETH_P_ALL));
    ifexit(s==-1,ERR_S,"Inizializzazione socket fallita");

    while (1) {                     /* ciclo per la ricezione dei pacchetti   */
        l=ricevi(s,buff,&prot);     /* ricezione del pacchetto                */
        if (prot==TIPO_IP) {        /* se il pacchetto e' di tipo IP          */
            if (!decidi(buff,&interface,&nextip))
                if (!cerca_mac((unsigned char *)&nextip,nextmac))
                    invia_arp(s,(unsigned char *)&nextip,interface);
                else
;//                    inoltra(s,interface,nextmac);
        }/* if */
        else if (prot==TIPO_ARP) {  /* se il pacchetto e' di tipo ARP         */
            /* estrazione coppia hw/prot address */
            if (elabora_arp((struct arp_packet *)buff,&ip,&mac)) {
                if (!cerca_mac(ip,mac))     /* se il record non viene trovato */
                    insert_arp(ip,mac);     /* memorizzazione del record      */
                stampa_at();        /* stampa la tabella ARP                  */
            }/* if */
        }/* else if */
    }/* while */
}/* main */


/* -------------------------------------------------------------------------- *
 * RICEVI: riceve un pacchetto e legge il tipo di protocollo                  *
 * -------------------------------------------------------------------------- *
 * I1: s    | identificativo dell'handler del socket                          *
 * O1: buff | buffer per la memorizzazione del pacchetto                      *
 * O2: prot | tipo di protocollo usato                                        *
 * O3: len  | dimensione del pacchetto ricevuto                               *
 * -------------------------------------------------------------------------- */
int ricevi (int s,char * buff,unsigned short *prot) {
    struct sockaddr_ll from;        /* link layer struct sockaddr             */
    int len,                        /* dimensione del buffer                  */
        from_len;                   /* dimensione della sockaddr_ll usata     */

    /* inizializzazione pacchetto per il passaggio alla "recvfrom"            */
    from.sll_family=AF_PACKET;      /* sempre impostata come AF_PACKET        */
    from.sll_protocol=TIPO_ALL;     /* tipo del protocollo di livello fisico  */
    from.sll_ifindex=0;             /* numero di interfaccia                  */
    from.sll_hatype=ARPHRD_ETHER;   /* tipo di indirizzo                      */
    from.sll_pkttype=PACKET_HOST;   /* tipo di pacchetto                      */
    from.sll_halen=0;               /* dimensione dell'indirizzo              */

	from_len=sizeof(struct sockaddr_ll); /* dimensione della struttura usata  */

    /* ricezione del pacchetto                                                */
    len=recvfrom(s,buff,L_BUFF,0,(struct sockaddr *)&from,&from_len);
    ifexit(len==-1,ERR_RCVF,"recvfrom fallita");

    *prot=from.sll_protocol;        /* lettura tipo di protocollo             */
    return len;
}/* ricevi */


/* -------------------------------------------------------------------------- *
 * ELABORA_ARP: restituisce l'indirizzo del sender/receiver                   *
 * -------------------------------------------------------------------------- *
 * I1: ap  | pt al pacchetto ARP da analizzare                                *
 * I2: ip  | pt a pt dell'indirizzo IP da restituire                          *
 * I3: mac | pt a pt dell'indirizzo MAC da restituire                         *
 * O : int | 0:IPs pari a 0.0.0.0, 1:coppia IP-MAC restituita                 *
 * -------------------------------------------------------------------------- */
int elabora_arp (struct arp_packet *ap,unsigned char **ip,unsigned char **mac) {
    if (ap->sender_pa[0]||ap->sender_pa[1]||      /* se l'IP non e' 0.0.0.0   */
        ap->sender_pa[2]||ap->sender_pa[3]) {
        *ip=ap->sender_pa;          /* copia dell'indirizzo ip                */
        *mac=ap->sender_ha;         /* copia dell'indirizzo mac               */
        return 1;                   /* ritorno esito positivo                 */
    }/* if */

    if (ap->receiver_pa[0]||ap->receiver_pa[1]||  /* se l'IP non e' 0.0.0.0   */
        ap->receiver_pa[2]||ap->receiver_pa[3]) {
        *ip=ap->receiver_pa;        /* copia dell'indirizzo ip                */
        *mac=ap->receiver_ha;       /* copia dell'indirizzo mac               */
        return 1;                   /* ritorno esito positivo                 */
    }/* if */

    return 0;                       /* ritorno esito negativo                 */
}/* elabora_arp */


/* -------------------------------------------------------------------------- *
 * INVIA_ARP:                                                                 *
 * -------------------------------------------------------------------------- *
 * I1: s   | identificativo dell'handler del socket                           *
 * I2: ip  | indirizzo IP                                                     *
 * I3: ifn | numero di interfaccia da usare                                   *
 * O : \   | \                                                                *
 * -------------------------------------------------------------------------- */
void invia_arp (int s,unsigned char *ip,int ifn) {
    struct sockaddr_ll to;
    struct arp_packet ap;

    to.sll_family=AF_PACKET;        /* sempre inpostata come AF_PACKET        */
    to.sll_protocol=TIPO_ARP;       /* tipo del protocollo di livello fisico  */
    to.sll_ifindex=ifn;             /* numero di interfaccia                  */
    to.sll_hatype=ARPHRD_ETHER;     /* tipo di indirizzo                      */
    /* impostazione dei campi dell'indirizzo al valore 255                    */
    *((unsigned int *)&(to.sll_addr))=0XFFFFFFFF;
    *(((unsigned int *)&(to.sll_addr))+1)=0XFFFFFFFF;
    to.sll_halen=6;                 /* dimensione dell'indirizzo (Ethernet=6) */

//    sendto(s,   ,&to,sizeof(struct sockaddr_ll));

    ap.hwtype=1;                    /* tipo della tecnologia di rete          */
    ap.prottype=TIPO_IP;            /* tipo di protocollo usato               */
    ap.hwsize=6;                    /* dimensione dell'indirizzo (Ethernet=6) */
    ap.protsize=4;                  /* dimensione del protocollo              */
    ap.op=htons(1);                 /* operazione da farsi (Request=1)        */
    /* impostazione dei campi dell'indirizzo MAC mittente al valore 0         */
    *((unsigned short int *)&(ap.sender_ha))=0;
    *(((unsigned short int *)&(ap.sender_ha))+2)=0;
//    ap.sender_pa[4];
//    ap.receiver_ha[6];
//    ap.receiver_pa[4];
}/* invia_arp */


/* -------------------------------------------------------------------------- *
 *                                                                            *
 * -------------------------------------------------------------------------- *
 * -------------------------------------------------------------------------- */
//void inoltra (int s,unsigned int interface,nextmac) {
//}/* inoltra */


/* -------------------------------------------------------------------------- *
 * IFEXIT: esce se la condizione passata e' vera                              *
 * -------------------------------------------------------------------------- *
 * I1: cond      | risultato della condizione passata (0:falso, 1:vero)       *
 * I2: exit_code | codice d'uscita da ritornare                               *
 * I3: msg       | messaggio da stampare                                      *
 * O : \         | \                                                          *
 * -------------------------------------------------------------------------- */
void ifexit (int cond,int exit_code,char msg[]) {
    if (cond) {                     /* se la condizione e' vera               */
        perror(msg);                /* stampa del messaggio d'errore          */
        exit(exit_code);            /* uscita dal programma                   */
    }/* if */
}/* ifexit */

