/* -------------------------------------------------------------------------- */
/* ! PRJ: router - router.h - file header per i files del progetto ---------- */
/* -------------------------------------------------------------------------- */


/* -------------------------------------------------------------------------- *
 * Costrutto per evitare la pluri compilazione di questo file header          *
 * -------------------------------------------------------------------------- */
#ifndef __ROUTER_H__
#define __ROUTER_H__


/* -------------------------------------------------------------------------- *
 * Headers delle funzioni da usare                                            *
 * -------------------------------------------------------------------------- */
#include <sys/socket.h>             /* funzioni per la creazione di sockets   */
#include <linux/if_arp.h>           /* defines, structs per il protocollo ARP */
#include <stdlib.h>                 /* exit(), abort(), ecc...                */
#include <stdio.h>                  /* standard I/O streams                   */
#include <features.h>               /* controllo versione librerie C/C++      */
#if __GLIBC__ >= 2 && __GLIBC_MINOR >= 1 /* se la versione e' >= alla 2.1     */
    #include <netpacket/packet.h>	/* strutture per i pacchetti, defines     */
    #include <net/ethernet.h>		/* headers addizionali, ecc...			  */
#else								/* la versione delle Glibc e' < alla 2.1  */
    #include <asm/types.h>			/* tipi di variabili addizionali          */
    #include <linux/if_packet.h>	/* strutture per i pacchetti, defines     */
    #include <linux/if_ether.h>		/* strutture per i pacchetti, defines     */
#endif


/* -------------------------------------------------------------------------- *
 * Defines                                                                    *
 * -------------------------------------------------------------------------- */
#define TIPO_ARP   htons(0x0806)    /* id del protocollo del tipo ARP         */
#define TIPO_IP    htons(0x0800)    /* id del protocollo del tipo IP          */
#define TIPO_ALL   htons(0x0003)	/* id che identifica tutti i tipi         */
#define L_TR       1000             /* dimensione della tabella di routing    */ 
#define L_AT       1000             /* dimensione della tabella ARP (MAC-IP)  */ 
#define L_BUFF     1500             /* dimensione massima del buffer usato    */
/* macro per convertire l'indirizzo IP in unsigned int dalla forma a.b.c.d    */
#define getIP(a,b,c,d) htonl( (unsigned long)a*0x1000000+b*0x10000+c*0x100+d )


/* -------------------------------------------------------------------------- *
 * Strutture                                                                  *
 * -------------------------------------------------------------------------- */
struct arp_packet {                 /* record del pacchetto arp               */
    unsigned short hwtype;          /* tipo della tecnologia di rete          */
    unsigned short prottype;        /* tipo di protocollo                     */
    unsigned char  hwsize;          /* dimensione dell'indirizzo (Ethernet=6) */
    unsigned char  protsize;        /* dimensione del protocollo              */
    unsigned short op;              /* operazione da farsi                    */
    unsigned char  sender_ha[6];    /* indirizzo MAC mittente                 */
    unsigned char  sender_pa[4];    /* indirizzo IP mittente                  */
    unsigned char  receiver_ha[6];  /* indirizzo MAC destinatario             */
    unsigned char  receiver_pa[4];  /* indirizzo IP destinatario              */
};/* struct arp_packet */

struct route_entry {                /* record della tabella di routing        */
    unsigned int   dest;            /* 4 bytes per l'indirizzo destinazione   */
    unsigned int   mask;            /* 4 bytes per l'indirizzo maschera       */
    unsigned int   nextip;          /* 4 bytes per il prossimo hop (salto)    */
    unsigned int   ifn;             /* numero di interfaccia                  */
};/* struct route_entry */

struct arp_entry {                  /* record tabella corrispondenze MAC-IP   */
    unsigned char  ip[4];           /* indirizzo IP                           */
    unsigned char  mac[6];          /* indirizzo MAC                          */
};/* struct arp_entry */

enum ERRS {                         /* enumerazione dei codici d'uscita       */
    ERR_S    = -1,                  /* inizializzazione socket fallita        */
    ERR_RCVF = -2                   /* recvfrom fallita                       */
};


/* -------------------------------------------------------------------------- *
 * Prototipi delle funzioni usate                                             *
 * -------------------------------------------------------------------------- */
int ricevi      (int s,char * buff,unsigned short *prot);
int elabora_arp (struct arp_packet * ap,unsigned char **ip,unsigned char **mac);
int decidi      (unsigned char *b,int *ifn,unsigned int *nextip);
int cerca_mac   (unsigned char *ip,unsigned char *mac);
void invia_arp  (int s,unsigned char *ip,int ifn);
//void inoltra    (int s,unsigned int interface,nextmac);
void ifexit     (int cond,int exit_code,char msg[]);
void insert_arp (unsigned char *ip,unsigned char *mac);
void stampa_at  ();
void init_rt    ();


/* -------------------------------------------------------------------------- *
 * Fine costrutto per evitare la pluri compilazione di questo file header     *
 * -------------------------------------------------------------------------- */
#endif //__ROUTER_H__

