/* -------------------------------------------------------------------------- */
/* ! PRJ: www - webserv.c - file principale --------------------------------- */
/* -------------------------------------------------------------------------- */


/* -------------------------------------------------------------------------- *
 * Headers delle funzioni da usare                                            *
 * -------------------------------------------------------------------------- */
#include <stdio.h>                  /* gestione dell'I/O                      */
#include <sys/socket.h>             /* gestione delle connessioni socket      */
#include <unistd.h>                 /* gestione dei file descriptor           */
#include <netinet/in.h>             /* gestione indirizzi internet            */
#include <string.h>                 /* gestione delle stringhe                */


/* -------------------------------------------------------------------------- *
 * Strutture                                                                  *
 * -------------------------------------------------------------------------- */
struct sockaddr_in ind_sv,			/* struttura per il socket del server     */
				   ind_cl;			/* struttura per il socket del client     */


/* -------------------------------------------------------------------------- *
 * Defines                                                                    *
 * -------------------------------------------------------------------------- */
#define LRISP 60000                 /* lunghezza massima della risposta       */


/* -------------------------------------------------------------------------- *
 * Prototipi delle funzioni usate                                             *
 * -------------------------------------------------------------------------- */
void ifexit (int cond,int exit_code,char msg[]);


/* -------------------------------------------------------------------------- *
 * MAIN: funzione principale                                                  *
 * -------------------------------------------------------------------------- *
 * I : \   | \                                                                *
 * O : int | codice di uscita dal programma                                   *
 * -------------------------------------------------------------------------- */
int main() {
    int  s_sv,                      /* handle del socket del server           */
         s_cl,                      /* handle del socket del client           */
         i,                         /* var contatore                          */
         tmp,                       /* var temporanea                         */
         n,                         /* n di bytes letti                       */
         ind_cl_size;               /* dimensione della struttura             */
    char risp[LRISP],               /* buffer della risposta del server       */
         *page_name;                /* puntatore al nome del file richiesto   */
    FILE *pf;                       /* puntatore al file da spedire           */

    /* apertura del socket per le ricezioni                                   */
    s_sv=socket(PF_INET,SOCK_STREAM,0);
    ifexit(s_sv==-1,-1,"Inizializzazione socket fallita");

    /* inizializzazione socket                                                */
    ind_sv.sin_family=AF_INET;
    ind_sv.sin_port=htons(8700);       /* porta dove attendere le richieste      */

    /* binding                                                                */
    tmp=bind(s_sv,(struct sockaddr *)&ind_sv,sizeof(struct sockaddr_in));
    ifexit(tmp!=0,-2,"Bind fallita");

    /* inizio attesa di connessioni                                           */
    tmp=listen(s_sv,SOMAXCONN);
    ifexit(tmp==-1,-3,"Listen fallita");

    while (1) {                     /* attesa di richieste all'infinito       */
        /* lettura del socket di una client in coda                           */
        s_cl=accept(s_sv,(struct sockaddr *)&ind_cl,&ind_cl_size);
        ifexit(s_cl==-1,-4,"Accept fallita");

        /* lettura del nome pagina richiesta                                  */
        n=read(s_cl,risp,LRISP-1);
        ifexit(n==-1,-5,"Lettura fallita");
        page_name=risp+5; /* page -> al primo carattere del nome della pagina */
        for (i=0;page_name[i]!=' ';i++) ;   /* lo spazio delimita il nome     */
        page_name[i]='\0';          /* delimitazione del nome                 */
        printf ("Nome del file richiesto: %s\n",page_name);

        /* se la pagina e' "getdate" leggi la data dal sistema in un file     */
        if (!strcmp(page_name,"getdate")) {
            system("date > data.htm");      /* lettura della data nel file    */
            strcpy(page_name,"data.htm");   /* modifica nome file da inviare  */
        }/* if */

        /* apertura del file da spedire                                       */
        if (!(pf=fopen(page_name,"rb"))) {  /* se il file non esiste          */
            perror("Errore lettura del file");
            sprintf(risp,"<html><head><title>Errore</title></head><body>"
						 "<center><br><hr width=\"80%\">Pagina <b>\"%s\""
						 "</b> non trovata<hr width=\"80%\"><center></bo"
						 "dy></html>",page_name);
            write(s_cl,risp,strlen(risp));  /* invio messaggio d'errore       */
        }/* if */
        else                                /* invio della pagina richiesta   */
            while (fread(risp,1,sizeof(char),pf)==1)
                write(s_cl,risp,1);

        /* chiusura della connessione client                                  */
        shutdown(s_cl,SHUT_RDWR);
        close(s_cl);
    }//while

	/* chiusura della connessione server									  */
	shutdown(s_sv,SHUT_RDWR);
    close(s_sv);

    return 0;                       /* programma terminato correttamente      */
}/* main */


/* -------------------------------------------------------------------------- *
 * IFEXIT: esce se la condizione passata e' vera                              *
 * -------------------------------------------------------------------------- *
 * I1: cond      | risultato della condizione passata (0:falso, 1:vero)       *
 * I2: exit_code | codice d'uscita da ritornare                               *
 * I3: msg       | messaggio da stampare                                      *
 * O : \         | \                                                          *
 * -------------------------------------------------------------------------- */
void ifexit (int cond,int exit_code,char msg[]) {
    if (cond) {                     /* se la condizione e' vera               */
        perror(msg);                /* stampa del messaggio d'errore          */
        exit(exit_code);            /* uscita dal programma                   */
    }/* if */
}/* ifexit */
